@@ -10,7 +10,9 @@
 # distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-"""Placeholder docstring"""
+"""This module contains functions to install and run the user-provided training
+entry point.
+"""
 from __future__ import absolute_import
 
 import os
@@ -20,7 +22,7 @@ from typing import Dict, List  # noqa ignore=F401 imported but unused
 
 from retrying import retry
 
-from sagemaker_training import _entry_point_type, env, files, modules, runner
+from sagemaker_training import _entry_point_type, environment, files, modules, runner
 
 
 def run(
@@ -34,8 +36,8 @@ def run(
     extra_opts=None,
 ):
     # type: (str, str, List[str], Dict[str, str], bool, bool, runner.RunnerType,Dict[str, str]) -> None  # pylint: disable=line-too-long # noqa ignore=E501
-    """Download, prepare and executes a compressed tar file from S3 or provided directory as an user
-    entrypoint. Runs the user entry point, passing env_vars as environment variables and args
+    """Download, prepare and execute a compressed tar file from S3 or provided directory as a user
+    entry point. Run the user entry point, passing env_vars as environment variables and args
     as command arguments.
 
     If the entry point is:
@@ -44,15 +46,14 @@ def run(
         - Any other: executes the command as >>> env_vars /bin/sh -c ./module_name + args
 
     Example:
-         >>>import sagemaker_training
-         >>>from sagemaker_training import entry_point
+         >>>from sagemaker_training import entry_point, environment, mapping
 
-         >>>env = sagemaker_training.training_env()
+         >>>env = environment.Environment()
          {'channel-input-dirs': {'training': '/opt/ml/input/training'},
           'model_dir': '/opt/ml/model', ...}
 
 
-         >>>hyperparameters = env.hyperparameters
+         >>>hyperparameters = environment.hyperparameters
          {'batch-size': 128, 'model_dir': '/opt/ml/model'}
 
          >>>args = mapping.to_cmd_args(hyperparameters)
@@ -68,29 +69,31 @@ def run(
                              --model_dir /opt/ml/model
 
     Args:
-        uri (str): the location of the module.
-        user_entry_point (str): name of the user provided entry point
+        uri (str): The location of the module or script. This can be an S3 uri, a path to
+            a local directory, or a path to a local tarball.
+        user_entry_point (str): Name of the user provided entry point.
         args (list):  A list of program arguments.
         env_vars (dict): A map containing the environment variables to be written (default: None).
         wait (bool): If the user entry point should be run to completion before this method returns
             (default: True).
         capture_error (bool): Default false. If True, the running process captures the
             stderr, and appends it to the returned Exception message in case of errors.
-        runner_type (sagemaker_training.runner.RunnerType): the type of runner object to
+        runner_type (sagemaker_training.runner.RunnerType): The type of runner object to
             be created (default: sagemaker_training.runner.ProcessRunnerType).
         extra_opts (dict): Additional options for running the entry point (default: None).
             Currently, this only applies for MPI.
 
     Returns:
-        sagemaker_training.process.ProcessRunner: the runner object responsible for
+        sagemaker_training.process.ProcessRunner: The runner object responsible for
             executing the entry point.
     """
     env_vars = env_vars or {}
     env_vars = env_vars.copy()
 
-    install(uri=uri, name=user_entry_point, path=env.code_dir, capture_error=capture_error)
+    files.download_and_extract(uri=uri, path=environment.code_dir)
+    install(name=user_entry_point, path=environment.code_dir, capture_error=capture_error)
 
-    env.write_env_vars(env_vars)
+    environment.write_env_vars(env_vars)
 
     _wait_hostname_resolution()
 
@@ -99,35 +102,37 @@ def run(
     )
 
 
-def install(uri, name=modules.DEFAULT_MODULE_NAME, path=env.code_dir, capture_error=False):
-    """Install the user provided entry point to be executed as follow:
+def install(name, path=environment.code_dir, capture_error=False):
+    """Install the user provided entry point to be executed as follows:
         - add the path to sys path
         - if the user entry point is a command, gives exec permissions to the script
 
     Args:
-        uri (str): the location of the module or script. This can be an S3 uri, a path to
-            a local directory, or a path to a local tarball.
-        name (str): name of the script or module.
-        path (str): path to directory where the entry point will be installed.
+        name (str): Name of the script or module.
+        path (str): Path to directory where the entry point will be installed.
         capture_error (bool): Default false. If True, the running process captures the
             stderr, and appends it to the returned Exception message in case of errors.
     """
-    files.download_and_extract(uri, path)
-
     if path not in sys.path:
         sys.path.insert(0, path)
 
     entry_point_type = _entry_point_type.get(path, name)
-    if entry_point_type is _entry_point_type.PYTHON_PACKAGE:
+
+    if (
+        entry_point_type is _entry_point_type.PYTHON_PACKAGE
+        or entry_point_type is _entry_point_type.PYTHON_PROGRAM
+        or modules.has_requirements(path)
+    ):
         modules.prepare(path, name)
         modules.install(path, capture_error)
+
     if entry_point_type is _entry_point_type.COMMAND:
         os.chmod(os.path.join(path, name), 511)
 
 
 @retry(stop_max_delay=1000 * 60 * 15, wait_exponential_multiplier=100, wait_exponential_max=30000)
 def _dns_lookup(host):
-    """ Retrying dns lookup on host """
+    """Retrying DNS lookup on host."""
     return socket.gethostbyname(host)
 
 
@@ -136,5 +141,5 @@ def _wait_hostname_resolution():
     boots up and has been documented here:
      https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms-training-algo-running-container.html#your-algorithms-training-algo-running-container-dist-training
     """
-    for host in env.TrainingEnv().hosts:
+    for host in environment.Environment().hosts:
         _dns_lookup(host)
